package spiderweb;

import java.util.ArrayList;

public class WebGene implements Comparable<WebGene> {
	public static final int GENE_LEN = 34;
	private int numberRadius;
	private ArrayList<Line> radiusLines;	// store all radius lines of web
	private ArrayList<Line> loopLines;	// store all loop lines of web
	public double fitness;
	
	// gene[0] : number of radiuses
	// gene[1] : bias angle
	// gene[2] ~ gene[11] : length of radius
	// gene[12] ~ gene[21] : angle of each
	// gene[22] ~ gene[31] : bias angle of each radiuses
	// gene[32] : d-start
	// gene[33] : d-end
	public int[] gene;
	
	WebGene() {
		this.radiusLines = new ArrayList<Line>();
		this.loopLines = new ArrayList<Line>();
		this.gene = new int[GENE_LEN];
		this.fitness = 0;
		this.numberRadius = 0;
	}
	
	private double generateWeb() {
		double silks = 0;
		radiusLines.clear();
		loopLines.clear();
		this.numberRadius = gene[0];
		double angleFixed = 360.0 / gene[0];
		double bias = gene[1];
		Point center = new Point(0, 0);
		
		// draw basic radiuses
		for (int i = 0; i < gene[0]; ++i) {
			double angle = bias + gene[22 + i];
			Point pt = new Point(gene[2 + i], 0);
			silks += gene[2 + i];
			radiusLines.add(new Line(new Point(0, 0), pt.rotate(center, angle)));
			//assert(Math.abs(pt.rotate(center, angle).norm() - gene[2 + i]) < 0.1);
			bias += angleFixed;
		}
		
		// draw addition radiuses
		for (int i = 0, j = 0; j < gene[0] - 1; ++j) {
			
			// draw radiuses between (i, i + 1) 
			int nxt = i + 1;
			Line first = radiusLines.get(i);
			Line second = radiusLines.get(i + 1);
			double diffAngle = first.getEnd().angle(second.getEnd());
			for (double angle = gene[12 + j]; angle < diffAngle; angle += gene[12 + j]) {
				Point pt = first.getEnd().rotate(center, angle);
				Point en = pt.intersect(first.getEnd(), second.getEnd());
				silks += en.norm();
				radiusLines.add(nxt, new Line(center, en));
				this.numberRadius++;
				nxt++;
			}
			i = nxt;
		}
		
		// draw radiuses between (n - 1, 0)
		{
			int nxt = radiusLines.size();
			Line first = radiusLines.get(nxt - 1);
			Line second = radiusLines.get(0);
			double diffAngle = first.getEnd().angle(second.getEnd());
			for (double angle = gene[12 + gene[0] - 1]; angle < diffAngle; angle += gene[12 + gene[0] - 1]) {
				Point pt = first.getEnd().rotate(center, angle);
				Point en = pt.intersect(first.getEnd(), second.getEnd());
				silks += en.norm();
				radiusLines.add(nxt, new Line(center, en));
				this.numberRadius++;
				nxt++;
			}
		}

		for (int i = 0; i < radiusLines.size(); ++i) {
			int nxt = (i + 1) % radiusLines.size();
			loopLines.add(new Line(radiusLines.get(i).getEnd(), radiusLines.get(nxt).getEnd()));
		}
		
		// draw outer loops
		double scale = Math.pow(gene[33] * 1.0 / gene[32], 1.0 / this.numberRadius);
		double currentLength = gene[32];
		Point now = radiusLines.get(0).getEnd().shrink(currentLength);
		double[] radiusPoint = new double[this.numberRadius];
		
		// draw first loop
		radiusPoint[0] = gene[32];
		for (int i = 0; i < this.numberRadius; ++i) {
			double nextLength = currentLength * scale;
			int nxt = (i + 1) % (this.numberRadius);
			Line nextLine = radiusLines.get(nxt);
			Point nextPoint = nextLine.getEnd().shrink(nextLength);
			if (nextPoint == null) return silks;
			loopLines.add(new Line(now, nextPoint));
			currentLength = nextLength;
			radiusPoint[nxt] = currentLength;
			silks += now.distance(nextPoint);
			now = nextPoint;
		}
		
		// draw rest of loops
		double delta = gene[33] - gene[32];
		int currentIndex = 0, step = 1;
		
		while (true) {
			int nextIndex = (currentIndex + step + this.numberRadius) % (this.numberRadius);
			double nextLength = radiusPoint[nextIndex] + delta;
			Line nextLine = radiusLines.get(nextIndex);
			if (nextLine.length() <= nextLength) {
				int preIndex = (currentIndex - step + this.numberRadius) % (this.numberRadius);
				Line previousLine = radiusLines.get(preIndex);
				nextLength = radiusPoint[preIndex] + delta;
				if (previousLine.length() <= nextLength) break;
				else step = -step;
			}
			else {
				Point nextPoint = nextLine.getEnd().shrink(nextLength);
				loopLines.add(new Line(now, nextPoint));
				now = nextPoint;
				currentLength = nextLength;
				currentIndex = nextIndex;
				radiusPoint[currentIndex] = currentLength;
				silks += now.distance(nextPoint);
			}
		}
		
		return silks;
	}
	
	double fitness(final Circle[] insects) {
		// create web and calculate total cost of silks
		double silks = generateWeb();
		
		// calculate number of catchable insects
		double ret = 0;
		for (int i = 0; i < insects.length; ++i) {
			boolean found = false;
			for (int j = 0; j < radiusLines.size(); ++j) {
				if (insects[i].isCross(radiusLines.get(j))) {
					ret++; found = true;
					break;
				}
			}
			if (!found) {
				for (int j = 0; j < loopLines.size(); ++j) {
					if (insects[i].isCross(loopLines.get(j))) {
						ret++; break;
					}
				}
			}
		}
		fitness = ret / silks;
		return fitness;
	}
	
	public String toSvg() {
		StringBuilder svg = new StringBuilder();
		double miniX = 0, miniY = 0;
		double maxiX = 0, maxiY = 0;
		
		for (int i = 0; i < radiusLines.size(); ++i) {
			Line ln = radiusLines.get(i);
			miniX = Math.min(miniX, ln.getEnd().getX());
			miniY = Math.min(miniY, ln.getEnd().getY());
			maxiX = Math.max(maxiX, ln.getEnd().getX());
			maxiY = Math.max(maxiY, ln.getEnd().getY());
		}
		
		int lx = (int) miniX - 1, ly = (int) miniY - 1;
		int rx = (int) maxiX + 1, ry = (int) maxiY + 1;
		int width = rx - lx, height = ry - ly;
		lx -= 100;
		ly -= 100;
		width += 100;
		height += 100;
		
		svg.append("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>" + "\n" + 
		"<svg xmlns=\"http://www.w3.org/2000/svg\" width=\"" + width + "\" height=\"" + height + "\" viewBox=\"" + (lx + " " + ly + " " + width + " " + height) + "\">\n");
		
		for (int i = 0; i < radiusLines.size(); ++i) {
			svg.append(radiusLines.get(i).toSvg());
			svg.append("\n");
		}
		for (int i = 0; i < loopLines.size(); ++i) {
			svg.append(loopLines.get(i).toSvg());
			svg.append("\n");
		}
		svg.append("</svg>");
		return svg.toString();
	}
	
	@SuppressWarnings("unchecked")
	public ArrayList<Line> getLines() {
		ArrayList<Line> ret = (ArrayList<Line>) this.radiusLines.clone();
		for (int i = 0; i < loopLines.size(); ++i) {
			ret.add(loopLines.get(i));
		}
		return ret;
	}

	public int compareTo(WebGene o) {
		// TODO Auto-generated method stub
		double diff = this.fitness - o.fitness;
		if (diff < 0) return 1;
		else if (diff > 0) return -1;
		else return 0;
	}
}